#include "LibEntryPoints.h"

double * EvalCurvature(HRBF_3D_r3::Vector & grad, HRBF_3D_r3::MatrixDD hes)
{
	double dfxx = hes(0, 0); double dfyx = hes(1, 0); double dfzx = hes(2, 0);
	double dfxy = hes(0, 1); double dfyy = hes(1, 1); double dfzy = hes(2, 1);
	double dfxz = hes(0, 2); double dfyz = hes(1, 2); double dfzz = hes(2, 2);

	//Eigen::MatrixXd HF(3, 3);
	Eigen::MatrixXd adjHF(3, 3);
	//Eigen::VectorXd gradF(3);


	//HF << dfxx, dfyx, dfzx,
	//	dfxy, dfyy, dfzy,
	//	dfxz, dfyz, dfzz;

	adjHF << dfyy*dfzz - dfyz*dfzy, dfxz*dfzy - dfxy*dfzz, dfxy*dfyz - dfxz*dfyy,
		dfyz*dfzx - dfyx*dfzz, dfxx*dfzz - dfxz*dfzx, dfyx*dfxz - dfxx*dfyz,
		dfyx*dfzy - dfyy*dfzx, dfxy*dfzx - dfxx*dfzy, dfxx*dfyy - dfxy*dfyx;


	double tmpx = (grad.transpose() * adjHF * grad);

	double gradFnorm = grad.norm();
	double gradFnorm2 = gradFnorm * gradFnorm;
	double K = (double)(grad.transpose() * adjHF * grad) / (gradFnorm2 * gradFnorm2); //gauss
	double H = ((grad.transpose() * hes * grad) - gradFnorm2 * hes.trace()) / (2 * gradFnorm * gradFnorm2); //mean

	double H2 = H * H;

	double K1, K2;

	if (abs(H2 - K) < H2 * 1E-13)
	{
		K1 = K2 = H;
	}
	else if (H2 < K)
	{
		K1 = 0;
		K2 = 0;
	}
	else
	{

		double sq = sqrt(H2 - K);
		K1 = H + sq;
		K2 = H - sq;
	}



	double * res = new double[2];
	res[0] = K1;
	res[1] = K2;

	return res;
};
